#include "Entity.h"
#include <math.h>

static const double PI = 3.14159265;
static const int MaxSpeedFactor = 10;
static const int MaxWheelsTurn = 5;

Entity::Entity(const TextureHolder &textures, Textures id, Category category, float x, float y, float angle)
	: mCategory(category)
	, mSpeedFactor(0)
	, mWheelsTurn(0)
{
	setTexture(textures.get(id));
	setScale(64.f / 300.f, 64.f / 300.f);
	sf::Vector2f pos(x, y);
	setPosition(pos);
	setRotation(angle);
}

void Entity::update(sf::Time elapsedTime, int maxX, int maxY)
{
	if (mSpeedFactor != 0) {
		// Rotate entity by taking care of mWheelsTurn
		rotate(0.05f * mWheelsTurn);

		// Move entity by taking care of mSpeedFactor
		double rotation = (double)(getRotation());
		sf::Vector2f movement(static_cast<float>(cos(rotation*PI / 180) * mSpeedFactor * 10),
			static_cast<float>(sin(rotation*PI / 180) * mSpeedFactor * 10));
		move(movement * elapsedTime.asSeconds());

		//
		// Adapt position and orientation if we have reached an edge
		//
		sf::Vector2f position = getPosition();
		sf::FloatRect boundingBox = getGlobalBounds();
		// Right of the screen
		if (((270 < rotation) || (rotation < 90)) && (position.x + boundingBox.width >= maxX)) {
			setRotation(180 - getRotation());
			if (rotation < 90) {
				sf::Vector2f offset(boundingBox.width, boundingBox.height);
				move(offset);
			}
			else {
				sf::Vector2f offset(boundingBox.width, 0);
				move(offset);
			}
		}
		// Bottom of the screen
		if (((0 < rotation) && (rotation < 180)) && (position.y + boundingBox.height > maxY)) {
			setRotation(360 - getRotation());
			if (rotation < 90) {
				sf::Vector2f offset(0, boundingBox.height);
				move(offset);
			}
			else {
				sf::Vector2f offset(-boundingBox.width, boundingBox.height);
				move(offset);
			}
		}
		// left of the screen
		if (((90 < rotation) && (rotation < 270)) && (position.x - boundingBox.width < 0)) {
			setRotation(180 - getRotation());
			if (rotation < 180) {
				sf::Vector2f offset(-boundingBox.width, 0);
				move(offset);
			}
			else {
				sf::Vector2f offset(-boundingBox.width, -boundingBox.height);
				move(offset);
			}
		}
		// Top of the screen
		if (((180 < rotation) && (rotation < 360)) && (position.y - boundingBox.height < 0)) {
			setRotation(360 - getRotation());
			if (rotation > 270) {
				sf::Vector2f offset(boundingBox.width, -boundingBox.height);
				move(offset);
			}
			else {
				sf::Vector2f offset(0, -boundingBox.height);
				move(offset);

			}
		}
	}

}

int	Entity::getSpeedFactor()
{
	return mSpeedFactor;
}

void Entity::speedFactor(int relativeSpeedFactor)
{
	if (((relativeSpeedFactor > 0) && (mSpeedFactor < MaxSpeedFactor)) ||
		((relativeSpeedFactor < 0) && (mSpeedFactor > 0))) {
		mSpeedFactor += relativeSpeedFactor;
	}
}

int	Entity::getWheelsTurn()
{
	return mWheelsTurn;
}

void Entity::wheelsTurn(int relativeWheelsTurn)
{
	if (((relativeWheelsTurn > 0) && (mWheelsTurn < MaxWheelsTurn)) ||
		((relativeWheelsTurn < 0) && (mWheelsTurn > -MaxWheelsTurn))) {
		mWheelsTurn += relativeWheelsTurn;
	}
}

